/*
 * $Id$
 * Copyright (c) 2011 Gresham Enterprise Storage (written by Frank Zago)
 *
 *	This software is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License as published by the Free Software Foundation; version 2.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public
 *	License along with this software; if not, write to the Free
 *	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <cf.h>
#include <sys/cfgodm.h>
#include <sys/cfgdb.h>
#include <errno.h>

#define MAX_ATTR 10
static char *attr_cl[MAX_ATTR];		/* attributes on command line */
static int num_attr_cl;					/* index for attr_cl */

static struct {
	char *name;
	char *value;
} attributes[MAX_ATTR];			/* extracted attributes */
static int num_attributes;

/* Attributes given with -a can be unique pairs, or a space-separated
 * list. Extract them in a nice array. */
static int build_attr_array()
{
	int i;

	for(i=0; i<num_attr_cl; i++) {
		char *str = strdup(attr_cl[i]);
		char *p1;
		char *p2;
		int has_next;

		/* Remove trailing space(s). */
		p1 = &str[strlen(str)-1];
		while (p1 != str && *p1==' ') {
			*p1=0;
			p1--;
		}

		p2 = str;
		
		do {
			if (num_attributes == MAX_ATTR)
				return E_ARGS;

			/* Get name. */
			p1 = strchr(p2, '=');
			if (!p1)
				return E_ARGS;

			*p1 = 0;
			attributes[num_attributes].name = strdup(p2);

			p1++;
			p2 = p1;

			/* Get value. */
			while(*p1 && *p1 != ' ')
				p1++;

			/* Is it the last value for this argument ? */
			if (*p1 == ' ')
				has_next = 1;
			else
				has_next = 0;

			*p1 = 0;
			attributes[num_attributes].value = strdup(p2);

			num_attributes ++;
			
			if (has_next)
				p2 = p1 + 1;
		} while(has_next);

		free(str);
	}

	return E_OK;
}

static void
err_exit(char exitcode)
{
    odm_close_class(CuDv_CLASS);
    odm_close_class(PdDv_CLASS);
    odm_terminate();
    exit(exitcode);
}

int main(int argc, char *argv[])
{
    char *parent, *connect, *lname;
    struct CuDv CuDv;
    int rc, errflg, c;
	int paramp;
	int i;
    char sstring[256];
    struct Class *cusdev;

	num_attr_cl = 0;
    errflg = 0;
    parent = NULL;
	connect = NULL;
	lname = 0;
	paramp = 0;

    while ((c = getopt(argc, argv, "l:p:w:Pa:")) != EOF) {
		switch (c) {
		case 'l':
			if (lname != NULL)
				errflg++;
			lname = optarg;
			break;
		case 'p':
			if (parent != NULL)
				errflg++;
			parent = optarg;
			break;
		case 'w':
			if (connect != NULL)
				errflg++;
			connect = optarg;
			break;
		case 'P':
			paramp = 1;
			break;
		case 'a':
			if (num_attr_cl == MAX_ATTR) {
				errflg++;
			} else {
				attr_cl[num_attr_cl] = optarg;
				num_attr_cl++;
			}
			break;
		default:
			errflg++;
		}
    }

    if (errflg) {
		exit(E_ARGS);
    }

	if (!lname) {
		exit(E_ARGS);
	}		

	/*
     * Open up ODM.
     */
    if (odm_initialize() == -1) {
		exit(E_ODMINIT);
    }
    if (odm_lock("/etc/objrepos/config_lock", 0) == -1) {
		err_exit(E_ODMLOCK);
    }

    /*
     * Open the Customized Device data base
     */
    if ((int) (cusdev = odm_open_class(CuDv_CLASS)) == -1) {
		err_exit(E_ODMOPEN);
    }

	/*
     * Get CuDv record
     */
    sprintf(sstring, "name = '%s'", lname);
    rc = (int) odm_get_first(cusdev, sstring, &CuDv);
    if (rc == 0) {
		err_exit(E_NOCuDv);
    } else if (rc == -1) {
		err_exit(E_ODMGET);
    }

	/* Validate all attributes */
	rc = build_attr_array();
	if (rc)
		err_exit(rc);

	for (i=0; i<num_attributes; i++) {
		if (!strcmp(attributes[i].name, "lun_id") ||
			!strcmp(attributes[i].name, "scsi_id")) {
			char *endptr;
			unsigned long long value;

			value = strtol(attributes[i].value, &endptr, 0);
			if ((errno == ERANGE && (value == ULLONG_MAX))
				|| (errno != 0 && value == 0)) {
				fprintf(stderr, "%s %s\n", attributes[i].name, attributes[i].value);
				err_exit(E_ATTRVAL);
			}

			if (endptr == attributes[i].value) {
				fprintf(stderr, "%s %s\n", attributes[i].name, attributes[i].value);
				err_exit(E_ATTRVAL);
			}
		
		} else {
			fprintf(stderr, "%s", attributes[i].name);
			err_exit(E_INVATTR);
		}
	}

	/* Note that we're missing a lot of steps from the documentation
	 * here. */

	/* Apply attributes changes. */
	for (i=0; i<num_attributes; i++) {
		struct CuAt *CuAt;
		int how_many;

		CuAt = getattr(CuDv.name, attributes[i].name, 0, &how_many);
		if (CuAt) {
			strcpy(CuAt->value, attributes[i].value);
			putattr(CuAt);
		}
	}

	err_exit(E_OK);
	return 0;
}
